import os.path

import pycodestyle
from testing.support import ROOT


def test_own_dog_food():
    style = pycodestyle.StyleGuide(select='E,W', quiet=True)
    files = [pycodestyle.__file__, __file__, os.path.join(ROOT, 'setup.py')]
    report = style.init_report(pycodestyle.StandardReport)
    report = style.check_files(files)
    assert list(report.messages) == ['W504'], f'Failures: {report.messages}'



import re

import pytest

import pycodestyle
from testing.support import errors_from_src

SELFTEST_REGEX = re.compile(r'\b(Okay|[EW]\d{3}): (.*)')


def get_tests():
    ret = [
        pytest.param(
            match[1],
            match[2],
            id=f'pycodestyle.py:{f.__code__.co_firstlineno}:{f.__name__}@{i}',
        )
        for group in pycodestyle._checks.values()
        for f in group
        if f.__doc__ is not None
        for i, match in enumerate(SELFTEST_REGEX.finditer(f.__doc__))
    ]
    assert ret
    return tuple(ret)


@pytest.mark.parametrize(('expected', 's'), get_tests())
def test(expected, s):
    s = '\n'.join((*s.replace(r'\t', '\t').split(r'\n'), ''))
    errors = errors_from_src(s)
    if expected == 'Okay':
        assert errors == []
    else:
        for error in errors:
            if error.startswith(f'{expected}:'):
                break
        else:
            raise AssertionError(f'expected {expected} from {s!r}')



from __future__ import annotations

import collections
import os.path
import re
import sys

import pytest

import pycodestyle
from testing.support import errors_from_src
from testing.support import ROOT

PY_RE = re.compile(r'^python(\d)(\d*)\.py$')
CASE_RE = re.compile('^(#:.*\n)', re.MULTILINE)


def _nsort(items: list[str]) -> list[str]:
    return sorted(
        items,
        key=lambda s: [
            int(part) if part.isdigit() else part.lower()
            for part in re.split(r'(\d+)', s)
        ],
    )


def get_tests():
    ret = []
    for fname in _nsort(os.listdir(os.path.join(ROOT, 'testing', 'data'))):
        match = PY_RE.match(fname)
        if match is not None:
            major, minor = int(match[1]), int(match[2] or '0')
            mark = pytest.mark.skipif(
                sys.version_info < (major, minor),
                reason=f'requires Python {major}.{minor}',
            )
        else:
            mark = ()

        fname = os.path.join('testing', 'data', fname)
        fname_full = os.path.join(ROOT, fname)
        src = ''.join(pycodestyle.readlines(fname_full))

        line = 1
        parts_it = iter(CASE_RE.split(src))
        # the first case will not have a comment for it
        s = next(parts_it)
        if s.strip():
            id_s = f'{fname}:{line}'
            ret.append(pytest.param('#: Okay', s, id=id_s, marks=mark))
            line += s.count('\n')

        for comment, s in zip(parts_it, parts_it):
            if s.strip():
                id_s = f'{fname}:{line}'
                ret.append(pytest.param(comment, s, id=id_s, marks=mark))
                line += s.count('\n') + 1

    assert ret
    return ret


@pytest.mark.parametrize(('case', 's'), get_tests())
def test(case, s):
    codes = collections.Counter()
    exact = collections.Counter()

    assert case.startswith('#:')
    for code in case[2:].strip().split():
        if code == 'Okay':
            continue
        elif code == 'noeol':
            s = s.rstrip('\n')
        elif ':' in code:
            exact[code] += 1
        else:
            codes[code] += 1

    unexpected = collections.Counter()
    for code in errors_from_src(s):
        if exact[code]:
            exact[code] -= 1
        elif codes[code[:4]]:
            codes[code[:4]] -= 1
        else:  # pragma: no cover
            unexpected[code] += 1

    messages = (
        *(f'-{k}\n' for k, v in codes.items() for _ in range(v)),
        *(f'-{k}\n' for k, v in exact.items() for _ in range(v)),
        *(f'+{k}\n' for k, v in unexpected.items() for _ in range(v)),
    )
    if messages:  # pragma: no cover
        raise AssertionError(f'unexpected codes!\n{"".join(messages)}')



import io
import os.path
import shlex
import sys
import unittest

import pycodestyle
from testing.support import ROOT

E11 = os.path.join(ROOT, 'testing', 'data', 'E11.py')


class DummyChecker:
    def __init__(self, tree, filename):
        pass

    def run(self):
        if False:
            yield


class APITestCase(unittest.TestCase):
    """Test the public methods."""

    def setUp(self):
        self._saved_stdout = sys.stdout
        self._saved_stderr = sys.stderr
        self._saved_checks = pycodestyle._checks
        sys.stdout = io.StringIO()
        sys.stderr = io.StringIO()
        pycodestyle._checks = {
            k: {f: (vals[0][:], vals[1]) for (f, vals) in v.items()}
            for k, v in self._saved_checks.items()
        }

    def tearDown(self):
        sys.stdout = self._saved_stdout
        sys.stderr = self._saved_stderr
        pycodestyle._checks = self._saved_checks

    def reset(self):
        sys.stdout.seek(0)
        sys.stdout.truncate()
        sys.stderr.seek(0)
        sys.stderr.truncate()

    def test_register_physical_check(self):
        def check_dummy(physical_line, line_number):
            raise NotImplementedError
        pycodestyle.register_check(check_dummy, ['Z001'])

        self.assertTrue(check_dummy in pycodestyle._checks['physical_line'])
        codes, args = pycodestyle._checks['physical_line'][check_dummy]
        self.assertTrue('Z001' in codes)
        self.assertEqual(args, ['physical_line', 'line_number'])

        options = pycodestyle.StyleGuide().options
        functions = [func for _, func, _ in options.physical_checks]
        self.assertIn(check_dummy, functions)

    def test_register_logical_check(self):
        def check_dummy(logical_line, tokens):
            raise NotImplementedError
        pycodestyle.register_check(check_dummy, ['Z401'])

        self.assertTrue(check_dummy in pycodestyle._checks['logical_line'])
        codes, args = pycodestyle._checks['logical_line'][check_dummy]
        self.assertTrue('Z401' in codes)
        self.assertEqual(args, ['logical_line', 'tokens'])

        pycodestyle.register_check(check_dummy, [])
        pycodestyle.register_check(check_dummy, ['Z402', 'Z403'])
        codes, args = pycodestyle._checks['logical_line'][check_dummy]
        self.assertEqual(codes, ['Z401', 'Z402', 'Z403'])
        self.assertEqual(args, ['logical_line', 'tokens'])

        options = pycodestyle.StyleGuide().options
        functions = [func for _, func, _ in options.logical_checks]
        self.assertIn(check_dummy, functions)

    def test_register_ast_check(self):
        pycodestyle.register_check(DummyChecker, ['Z701'])

        self.assertTrue(DummyChecker in pycodestyle._checks['tree'])
        codes, args = pycodestyle._checks['tree'][DummyChecker]
        self.assertTrue('Z701' in codes)
        self.assertTrue(args is None)

        options = pycodestyle.StyleGuide().options
        classes = [cls for _, cls, _ in options.ast_checks]
        self.assertIn(DummyChecker, classes)

    def test_register_invalid_check(self):
        class InvalidChecker(DummyChecker):
            def __init__(self, filename):
                raise NotImplementedError

        def check_dummy(logical, tokens):
            raise NotImplementedError

        pycodestyle.register_check(InvalidChecker, ['Z741'])
        pycodestyle.register_check(check_dummy, ['Z441'])

        for checkers in pycodestyle._checks.values():
            self.assertTrue(DummyChecker not in checkers)
            self.assertTrue(check_dummy not in checkers)

        self.assertRaises(TypeError, pycodestyle.register_check)

    def test_styleguide(self):
        report = pycodestyle.StyleGuide().check_files()
        self.assertEqual(report.total_errors, 0)
        self.assertFalse(sys.stdout.getvalue())
        self.assertFalse(sys.stderr.getvalue())
        self.reset()

        report = pycodestyle.StyleGuide().check_files(['missing-file'])
        stdout = sys.stdout.getvalue().splitlines()
        self.assertEqual(len(stdout), report.total_errors)
        self.assertEqual(report.total_errors, 1)
        # < 3.3 returns IOError; >= 3.3 returns FileNotFoundError
        assert stdout[0].startswith("missing-file:1:1: E902 ")
        self.assertFalse(sys.stderr.getvalue())
        self.reset()

        report = pycodestyle.StyleGuide().check_files([E11])
        stdout = sys.stdout.getvalue().splitlines()
        self.assertEqual(len(stdout), report.total_errors)
        self.assertEqual(report.total_errors, 24)
        self.assertFalse(sys.stderr.getvalue())
        self.reset()

        # Passing the paths in the constructor gives same result
        report = pycodestyle.StyleGuide(paths=[E11]).check_files()
        stdout = sys.stdout.getvalue().splitlines()
        self.assertEqual(len(stdout), report.total_errors)
        self.assertEqual(report.total_errors, 24)
        self.assertFalse(sys.stderr.getvalue())
        self.reset()

    def test_styleguide_options(self):
        # Instantiate a simple checker
        pep8style = pycodestyle.StyleGuide(paths=[E11])

        # Check style's attributes
        self.assertEqual(pep8style.checker_class, pycodestyle.Checker)
        self.assertEqual(pep8style.paths, [E11])
        self.assertEqual(pep8style.runner, pep8style.input_file)
        self.assertEqual(pep8style.options.ignore_code, pep8style.ignore_code)
        self.assertEqual(pep8style.options.paths, pep8style.paths)

        # Check unset options
        for o in ('benchmark', 'config', 'count', 'diff',
                  'quiet', 'show_pep8', 'show_source',
                  'statistics', 'verbose'):
            oval = getattr(pep8style.options, o)
            self.assertTrue(oval in (None, False), msg=f'{o} = {oval!r}')

        # Check default options
        self.assertTrue(pep8style.options.repeat)
        self.assertEqual(pep8style.options.benchmark_keys,
                         ['directories', 'files',
                          'logical lines', 'physical lines'])
        self.assertEqual(pep8style.options.exclude,
                         ['.svn', 'CVS', '.bzr', '.hg',
                          '.git', '__pycache__', '.tox'])
        self.assertEqual(pep8style.options.filename, ['*.py'])
        self.assertEqual(pep8style.options.format, 'default')
        self.assertEqual(pep8style.options.select, ())
        self.assertEqual(pep8style.options.ignore, ('E226', 'E24', 'W504'))
        self.assertEqual(pep8style.options.max_line_length, 79)

    def test_styleguide_ignore_code(self):
        def parse_argv(argstring):
            _saved_argv = sys.argv
            sys.argv = shlex.split('pycodestyle %s /dev/null' % argstring)
            try:
                return pycodestyle.StyleGuide(parse_argv=True)
            finally:
                sys.argv = _saved_argv

        options = parse_argv('').options
        self.assertEqual(options.select, ())
        self.assertEqual(
            options.ignore,
            ('E121', 'E123', 'E126', 'E226', 'E24', 'E704', 'W503', 'W504')
        )

        options = parse_argv('--ignore E,W').options
        self.assertEqual(options.select, ())
        self.assertEqual(options.ignore, ('E', 'W'))

        options = parse_argv('--select E,W').options
        self.assertEqual(options.select, ('E', 'W'))
        self.assertEqual(options.ignore, ('',))

        options = parse_argv('--select E --ignore E24').options
        self.assertEqual(options.select, ('E',))
        self.assertEqual(options.ignore, ('',))

        options = parse_argv('--ignore E --select E24').options
        self.assertEqual(options.select, ('E24',))
        self.assertEqual(options.ignore, ('',))

        options = parse_argv('--ignore W --select E24').options
        self.assertEqual(options.select, ('E24',))
        self.assertEqual(options.ignore, ('',))

        options = parse_argv('--max-doc-length=72').options
        self.assertEqual(options.max_doc_length, 72)

        options = parse_argv('').options
        self.assertEqual(options.max_doc_length, None)

        pep8style = pycodestyle.StyleGuide(paths=[E11])
        self.assertFalse(pep8style.ignore_code('E112'))
        self.assertFalse(pep8style.ignore_code('W191'))
        self.assertTrue(pep8style.ignore_code('E241'))

        pep8style = pycodestyle.StyleGuide(select='E', paths=[E11])
        self.assertFalse(pep8style.ignore_code('E112'))
        self.assertTrue(pep8style.ignore_code('W191'))
        self.assertFalse(pep8style.ignore_code('E241'))

        pep8style = pycodestyle.StyleGuide(select='W', paths=[E11])
        self.assertTrue(pep8style.ignore_code('E112'))
        self.assertFalse(pep8style.ignore_code('W191'))
        self.assertTrue(pep8style.ignore_code('E241'))

        pep8style = pycodestyle.StyleGuide(select=('F401',), paths=[E11])
        self.assertEqual(pep8style.options.select, ('F401',))
        self.assertEqual(pep8style.options.ignore, ('',))
        self.assertFalse(pep8style.ignore_code('F'))
        self.assertFalse(pep8style.ignore_code('F401'))
        self.assertTrue(pep8style.ignore_code('F402'))

    def test_styleguide_excluded(self):
        pep8style = pycodestyle.StyleGuide(paths=[E11])

        self.assertFalse(pep8style.excluded('./foo/bar'))
        self.assertFalse(pep8style.excluded('./foo/bar/main.py'))

        self.assertTrue(pep8style.excluded('./CVS'))
        self.assertTrue(pep8style.excluded('./.tox'))
        self.assertTrue(pep8style.excluded('./subdir/CVS'))
        self.assertTrue(pep8style.excluded('__pycache__'))
        self.assertTrue(pep8style.excluded('./__pycache__'))
        self.assertTrue(pep8style.excluded('subdir/__pycache__'))

        self.assertFalse(pep8style.excluded('draftCVS'))
        self.assertFalse(pep8style.excluded('./CVSoup'))
        self.assertFalse(pep8style.excluded('./CVS/subdir'))

    def test_styleguide_checks(self):
        pep8style = pycodestyle.StyleGuide(paths=[E11])

        # Default lists of checkers
        self.assertTrue(len(pep8style.options.physical_checks) > 4)
        self.assertTrue(len(pep8style.options.logical_checks) > 10)
        self.assertEqual(len(pep8style.options.ast_checks), 0)

        # Sanity check
        for name, check, args in pep8style.options.physical_checks:
            self.assertEqual(check.__name__, name)
            self.assertEqual(args[0], 'physical_line')
        for name, check, args in pep8style.options.logical_checks:
            self.assertEqual(check.__name__, name)
            self.assertEqual(args[0], 'logical_line')

        # Do run E11 checks
        options = pycodestyle.StyleGuide().options
        functions = [func for _, func, _ in options.logical_checks]
        self.assertIn(pycodestyle.indentation, functions)
        options = pycodestyle.StyleGuide(select=['E']).options
        functions = [func for _, func, _ in options.logical_checks]
        self.assertIn(pycodestyle.indentation, functions)
        options = pycodestyle.StyleGuide(ignore=['W']).options
        functions = [func for _, func, _ in options.logical_checks]
        self.assertIn(pycodestyle.indentation, functions)
        options = pycodestyle.StyleGuide(ignore=['E12']).options
        functions = [func for _, func, _ in options.logical_checks]
        self.assertIn(pycodestyle.indentation, functions)

        # Do not run E11 checks
        options = pycodestyle.StyleGuide(select=['W']).options
        functions = [func for _, func, _ in options.logical_checks]
        self.assertNotIn(pycodestyle.indentation, functions)
        options = pycodestyle.StyleGuide(ignore=['E']).options
        functions = [func for _, func, _ in options.logical_checks]
        self.assertNotIn(pycodestyle.indentation, functions)
        options = pycodestyle.StyleGuide(ignore=['E11']).options
        functions = [func for _, func, _ in options.logical_checks]
        self.assertNotIn(pycodestyle.indentation, functions)

    def test_styleguide_init_report(self):
        style = pycodestyle.StyleGuide(paths=[E11])

        standard_report = pycodestyle.StandardReport

        self.assertEqual(style.options.reporter, standard_report)
        self.assertEqual(type(style.options.report), standard_report)

        class MinorityReport(pycodestyle.BaseReport):
            pass

        report = style.init_report(MinorityReport)
        self.assertEqual(style.options.report, report)
        self.assertEqual(type(report), MinorityReport)

        style = pycodestyle.StyleGuide(paths=[E11], reporter=MinorityReport)
        self.assertEqual(type(style.options.report), MinorityReport)
        self.assertEqual(style.options.reporter, MinorityReport)

    def test_styleguide_check_files(self):
        pep8style = pycodestyle.StyleGuide(paths=[E11])

        report = pep8style.check_files()
        self.assertTrue(report.total_errors)

        self.assertRaises(TypeError, pep8style.check_files, 42)
        # < 3.3 raises TypeError; >= 3.3 raises AttributeError
        self.assertRaises(Exception, pep8style.check_files, [42])

    def test_check_nullbytes(self):
        pycodestyle.register_check(DummyChecker, ['Z701'])

        pep8style = pycodestyle.StyleGuide()
        count_errors = pep8style.input_file('stdin', lines=['\x00\n'])

        stdout = sys.stdout.getvalue()
        if sys.version_info < (3, 11, 4):  # pragma: <3.11 cover
            expected = ["stdin:1:1: E901 ValueError: source code string cannot contain null bytes"]  # noqa: E501
        elif sys.version_info < (3, 12):  # pragma: <3.12 cover  # pragma: >=3.11 cover  # noqa: E501
            expected = ["stdin:1:1: E901 SyntaxError: source code string cannot contain null bytes"]  # noqa: E501
        else:  # pragma: >=3.12 cover
            expected = [
                "stdin:1:1: E901 SyntaxError: source code string cannot contain null bytes",   # noqa: E501
                "stdin:1:1: E901 TokenError: source code cannot contain null bytes",   # noqa: E501
            ]
        self.assertEqual(stdout.splitlines(), expected)
        self.assertFalse(sys.stderr.getvalue())
        self.assertEqual(count_errors, len(expected))

    def test_styleguide_unmatched_triple_quotes(self):
        pycodestyle.register_check(DummyChecker, ['Z701'])
        lines = [
            'def foo():\n',
            '    """test docstring""\'\n',
        ]

        pep8style = pycodestyle.StyleGuide()
        pep8style.input_file('stdin', lines=lines)
        stdout = sys.stdout.getvalue()

        if sys.version_info < (3, 10):  # pragma: <3.10 cover
            expected = [
                'stdin:2:5: E901 TokenError: EOF in multi-line string',
                'stdin:2:26: E901 SyntaxError: EOF while scanning triple-quoted string literal',  # noqa: E501
            ]
        elif sys.version_info < (3, 12):  # pragma: >=3.10 cover  # pragma: <3.12 cover  # noqa: E501
            expected = [
                'stdin:2:5: E901 TokenError: EOF in multi-line string',
                'stdin:2:6: E901 SyntaxError: unterminated triple-quoted string literal (detected at line 2)',  # noqa: E501
            ]
        else:  # pragma: >=3.12 cover
            expected = [
                'stdin:2:6: E901 SyntaxError: unterminated triple-quoted string literal (detected at line 2)',  # noqa: E501
                'stdin:2:6: E901 TokenError: EOF in multi-line string',
            ]
        self.assertEqual(stdout.splitlines(), expected)

    def test_styleguides_other_indent_size(self):
        pycodestyle.register_check(DummyChecker, ['Z701'])
        lines = [
            'def foo():\n',
            '    pass\n',
            '\n',
            '\n',
            'def foo_correct():\n',
            '   pass\n',
            '\n',
            '\n',
            'def bar():\n',
            '   [1, 2, 3,\n',
            '     4, 5, 6,\n',
            '     ]\n',
            '\n',
            '\n',
            'if (1 in [1, 2, 3]\n',
            '      and bool(0) is False\n',
            '     and bool(1) is True):\n',
            '   pass\n'
        ]

        pep8style = pycodestyle.StyleGuide()
        pep8style.options.indent_size = 3
        count_errors = pep8style.input_file('stdin', lines=lines)
        stdout = sys.stdout.getvalue()
        self.assertEqual(count_errors, 4)
        expected = (
            'stdin:2:5: '
            'E111 indentation is not a multiple of 3'
        )
        self.assertTrue(expected in stdout)
        expected = (
            'stdin:11:6: '
            'E127 continuation line over-indented for visual indent'
        )
        self.assertTrue(expected in stdout)
        expected = (
            'stdin:12:6: '
            'E124 closing bracket does not match visual indentation'
        )
        self.assertTrue(expected in stdout)
        expected = (
            'stdin:17:6: '
            'E127 continuation line over-indented for visual indent'
        )
        self.assertTrue(expected in stdout)



import io
import sys
import tokenize

import pytest

from pycodestyle import Checker
from pycodestyle import expand_indent
from pycodestyle import mute_string


@pytest.mark.parametrize(
    ('s', 'expected'),
    (
        ('    ', 4),
        ('\t', 8),
        ('       \t', 8),
        ('        \t', 16),
    ),
)
def test_expand_indent(s, expected):
    assert expand_indent(s) == expected


@pytest.mark.parametrize(
    ('s', 'expected'),
    (
        ('"abc"', '"xxx"'),
        ("'''abc'''", "'''xxx'''"),
        ("r'abc'", "r'xxx'"),
    ),
)
def test_mute_string(s, expected):
    assert mute_string(s) == expected


def test_fstring_logical_line():
    src = '''\
f'hello {{ {thing} }} world'
'''
    checker = Checker(lines=src.splitlines())
    checker.tokens = list(tokenize.generate_tokens(io.StringIO(src).readline))
    checker.build_tokens_line()

    if sys.version_info >= (3, 12):  # pragma: >3.12 cover
        assert checker.logical_line == "f'xxxxxxxxx{thing}xxxxxxxxx'"
    else:
        assert checker.logical_line == "f'xxxxxxxxxxxxxxxxxxxxxxxxx'"



"""moved from data files due to 3.12 changing syntax errors"""
import sys
import unittest

from testing.support import errors_from_src


class E901Test(unittest.TestCase):
    def test_closing_brace(self):
        errors = errors_from_src('}\n')
        if sys.version_info < (3, 12):  # pragma: <3.12 cover
            self.assertEqual(errors, ['E901:2:1'])
        else:  # pragma: >=3.12 cover
            self.assertEqual(errors, [])

    def test_unclosed_brace(self):
        src = '''\
if msg:
    errmsg = msg % progress.get(cr_dbname))

def lasting(self, duration=300):
    progress = self._progress.setdefault('foo', {}
'''
        errors = errors_from_src(src)
        if sys.version_info < (3, 12):  # pragma: <3.12 cover
            expected = ['E122:4:1']
        else:  # pragma: >=3.12 cover
            expected = ['E122:4:1', 'E901:5:1']  # noqa: E501
        self.assertEqual(errors, expected)



import os
import tempfile
import unittest

import pycodestyle


def _process_file(contents):
    with tempfile.NamedTemporaryFile(delete=False) as f:
        f.write(contents)

    options, args = pycodestyle.process_options(config_file=f.name)
    os.remove(f.name)

    return options, args


class ParserTestCase(unittest.TestCase):

    def test_vanilla_ignore_parsing(self):
        contents = b"""
[pycodestyle]
ignore = E226,E24
        """
        options, args = _process_file(contents)

        self.assertEqual(options.ignore, ["E226", "E24"])

    def test_multiline_ignore_parsing(self):
        contents = b"""
[pycodestyle]
ignore =
    E226,
    E24
        """

        options, args = _process_file(contents)

        self.assertEqual(options.ignore, ["E226", "E24"])

    def test_trailing_comma_ignore_parsing(self):
        contents = b"""
[pycodestyle]
ignore = E226,
        """

        options, args = _process_file(contents)

        self.assertEqual(options.ignore, ["E226"])

    def test_multiline_trailing_comma_ignore_parsing(self):
        contents = b"""
[pycodestyle]
ignore =
    E226,
    E24,
        """

        options, args = _process_file(contents)

        self.assertEqual(options.ignore, ["E226", "E24"])



import os
import unittest

from pycodestyle import normalize_paths


class UtilTestCase(unittest.TestCase):
    def test_normalize_paths(self):
        self.assertEqual(normalize_paths(''), [])
        self.assertEqual(normalize_paths([]), [])
        self.assertEqual(normalize_paths(None), [])
        self.assertEqual(normalize_paths(['foo']), ['foo'])
        self.assertEqual(normalize_paths('foo'), ['foo'])
        self.assertEqual(normalize_paths('foo,bar'), ['foo', 'bar'])
        self.assertEqual(normalize_paths('foo,  bar  '), ['foo', 'bar'])
        self.assertEqual(
            normalize_paths('/foo/bar,baz/../bat'),
            [os.path.realpath('/foo/bar'), os.path.abspath('bat')],
        )
        self.assertEqual(
            normalize_paths(".pyc,\n   build/*"),
            ['.pyc', os.path.abspath('build/*')],
        )



"""
Tests for the blank_lines checker.

It uses dedicated assertions which work with TestReport.
"""
import unittest

import pycodestyle
from testing.support import errors_from_src


class BlankLinesTestCase(unittest.TestCase):
    """
    Common code for running blank_lines tests.
    """

    def assertNoErrors(self, actual):
        """
        Check that the actual result from the checker has no errors.
        """
        self.assertEqual([], actual)


class TestBlankLinesDefault(BlankLinesTestCase):
    """
    Tests for default blank with 2 blank lines for top level and 1
    blank line for methods.
    """

    def test_initial_no_blank(self):
        """
        It will accept no blank lines at the start of the file.
        """
        result = errors_from_src("""def some_function():
    pass
""")

        self.assertNoErrors(result)

    def test_initial_lines_one_blank(self):
        """
        It will accept 1 blank lines before the first line of actual
        code, even if in other places it asks for 2
        """
        result = errors_from_src("""
def some_function():
    pass
""")

        self.assertNoErrors(result)

    def test_initial_lines_two_blanks(self):
        """
        It will accept 2 blank lines before the first line of actual
        code, as normal.
        """
        result = errors_from_src("""

def some_function():
    pass
""")

        self.assertNoErrors(result)

    def test_method_less_blank_lines(self):
        """
        It will trigger an error when less than 1 blank lin is found
        before method definitions.
        """
        result = errors_from_src("""# First comment line.
class X:

    def a():
        pass
    def b():
        pass
""")
        self.assertEqual([
            'E301:6:5',  # b() call
        ], result)

    def test_method_less_blank_lines_comment(self):
        """
        It will trigger an error when less than 1 blank lin is found
        before method definition, ignoring comments.
        """
        result = errors_from_src("""# First comment line.
class X:

    def a():
        pass
    # A comment will not make it better.
    def b():
        pass
""")
        self.assertEqual([
            'E301:7:5',  # b() call
        ], result)

    def test_top_level_fewer_blank_lines(self):
        """
        It will trigger an error when less 2 blank lines are found
        before top level definitions.
        """
        result = errors_from_src("""# First comment line.
# Second line of comment.

def some_function():
    pass

async def another_function():
    pass


def this_one_is_good():
    pass

class SomeCloseClass(object):
    pass


async def this_async_is_good():
    pass


class AFarEnoughClass(object):
    pass
""")
        self.assertEqual([
            'E302:7:1',  # another_function
            'E302:14:1',  # SomeCloseClass
        ], result)

    def test_top_level_more_blank_lines(self):
        """
        It will trigger an error when more 2 blank lines are found
        before top level definitions.
        """
        result = errors_from_src("""# First comment line.
# Second line of comment.



def some_function():
    pass


def this_one_is_good():
    pass



class SomeFarClass(object):
    pass


class AFarEnoughClass(object):
    pass
""")
        self.assertEqual([
            'E303:6:1',  # some_function
            'E303:15:1',  # SomeFarClass
        ], result)

    def test_method_more_blank_lines(self):
        """
        It will trigger an error when more than 1 blank line is found
        before method definition
        """
        result = errors_from_src("""# First comment line.


class SomeCloseClass(object):


    def oneMethod(self):
        pass


    def anotherMethod(self):
        pass

    def methodOK(self):
        pass



    def veryFar(self):
        pass
""")
        self.assertEqual([
            'E303:7:5',  # oneMethod
            'E303:11:5',  # anotherMethod
            'E303:19:5',  # veryFar
        ], result)

    def test_initial_lines_more_blank(self):
        """
        It will trigger an error for more than 2 blank lines before the
        first line of actual code.
        """
        result = errors_from_src("""


def some_function():
    pass
""")
        self.assertEqual(['E303:4:1'], result)

    def test_blank_line_between_decorator(self):
        """
        It will trigger an error when the decorator is followed by a
        blank line.
        """
        result = errors_from_src("""# First line.


@some_decorator

def some_function():
    pass


class SomeClass(object):

    @method_decorator

    def some_method(self):
        pass
""")
        self.assertEqual(['E304:6:1', 'E304:14:5'], result)

    def test_blank_line_decorator(self):
        """
        It will accept the decorators which are adjacent to the function
        and method definition.
        """
        result = errors_from_src("""# First line.


@another_decorator
@some_decorator
def some_function():
    pass


class SomeClass(object):

    @method_decorator
    def some_method(self):
        pass
""")
        self.assertNoErrors(result)

    def test_top_level_fewer_follow_lines(self):
        """
        It will trigger an error when less than 2 blank lines are
        found between a top level definitions and other top level code.
        """
        result = errors_from_src("""
def a():
    print('Something')

a()
""")
        self.assertEqual([
            'E305:5:1',  # a call
        ], result)

    def test_top_level_fewer_follow_lines_comments(self):
        """
        It will trigger an error when less than 2 blank lines are
        found between a top level definitions and other top level code,
        even if we have comments before
        """
        result = errors_from_src("""
def a():
    print('Something')

    # comment

    # another comment

# With comment still needs 2 spaces before,
# as comments are ignored.
a()
""")
        self.assertEqual([
            'E305:11:1',  # a call
        ], result)

    def test_top_level_good_follow_lines(self):
        """
        It not trigger an error when 2 blank lines are
        found between a top level definitions and other top level code.
        """
        result = errors_from_src("""
def a():
    print('Something')

    # Some comments in other parts.

    # More comments.


# With the right spaces,
# It will work, even when we have comments.
a()
""")
        self.assertNoErrors(result)

    def test_method_fewer_follow_lines(self):
        """
        It will trigger an error when less than 1 blank line is
        found between a method and previous definitions.
        """
        result = errors_from_src("""
def a():
    x = 1
    def b():
        pass
""")
        self.assertEqual([
            'E306:4:5',  # b() call
        ], result)

    def test_method_nested_fewer_follow_lines(self):
        """
        It will trigger an error when less than 1 blank line is
        found between a method and previous definitions, even when
        nested.
        """
        result = errors_from_src("""
def a():
    x = 2

    def b():
        x = 1
        def c():
            pass
""")
        self.assertEqual([
            'E306:7:9',  # c() call
        ], result)

    def test_method_nested_less_class(self):
        """
        It will trigger an error when less than 1 blank line is found
        between a method and previous definitions, even when used to
        define a class.
        """
        result = errors_from_src("""
def a():
    x = 1
    class C:
        pass
""")
        self.assertEqual([
            'E306:4:5',  # class C definition.
        ], result)

    def test_method_nested_ok(self):
        """
        Will not trigger an error when 1 blank line is found
        found between a method and previous definitions, even when
        nested.
        """
        result = errors_from_src("""
def a():
    x = 2

    def b():
        x = 1

        def c():
            pass

    class C:
        pass
""")
        self.assertNoErrors(result)


class TestBlankLinesTwisted(BlankLinesTestCase):
    """
    Tests for blank_lines with 3 blank lines for top level and 2 blank
    line for methods as used by the Twisted coding style.
    """

    def setUp(self):
        self._original_lines_config = pycodestyle.BLANK_LINES_CONFIG.copy()
        pycodestyle.BLANK_LINES_CONFIG['top_level'] = 3
        pycodestyle.BLANK_LINES_CONFIG['method'] = 2

    def tearDown(self):
        pycodestyle.BLANK_LINES_CONFIG = self._original_lines_config

    def test_initial_lines_one_blanks(self):
        """
        It will accept less than 3 blank lines before the first line of
        actual code.
        """
        result = errors_from_src("""


def some_function():
    pass
""")

        self.assertNoErrors(result)

    def test_initial_lines_tree_blanks(self):
        """
        It will accept 3 blank lines before the first line of actual
        code, as normal.
        """
        result = errors_from_src("""


def some_function():
    pass
""")

        self.assertNoErrors(result)

    def test_top_level_fewer_blank_lines(self):
        """
        It will trigger an error when less 3 blank lines are found
        before top level definitions.
        """
        result = errors_from_src("""# First comment line.
# Second line of comment.


def some_function():
    pass


async def another_function():
    pass



def this_one_is_good():
    pass

class SomeCloseClass(object):
    pass



async def this_async_is_good():
    pass



class AFarEnoughClass(object):
    pass
""")
        self.assertEqual([
            'E302:9:1',  # another_function
            'E302:17:1',  # SomeCloseClass
        ], result)

    def test_top_level_more_blank_lines(self):
        """
        It will trigger an error when more 2 blank lines are found
        before top level definitions.
        """
        result = errors_from_src("""# First comment line.
# Second line of comment.




def some_function():
    pass



def this_one_is_good():
    pass




class SomeVeryFarClass(object):
    pass



class AFarEnoughClass(object):
    pass
""")
        self.assertEqual([
            'E303:7:1',  # some_function
            'E303:18:1',  # SomeVeryFarClass
        ], result)

    def test_the_right_blanks(self):
        """
        It will accept 3 blank for top level and 2 for nested.
        """
        result = errors_from_src("""


def some_function():
    pass



# With comments.
some_other = code_here



class SomeClass:
    '''
    Docstring here.
    '''

    def some_method():
        pass


    def another_method():
        pass


    # More methods.
    def another_method_with_comment():
        pass


    @decorator
    def another_method_with_comment():
        pass
""")

        self.assertNoErrors(result)



"""moved from data files due to 3.12 making this a TokenError"""
import sys
import unittest

from testing.support import errors_from_src


class E101Test(unittest.TestCase):
    def test_E101(self):
        errors = errors_from_src(
            'if True:\n'
            '\tprint(1)  # tabs\n'
            '        print(2)  # spaces\n'
        )
        if sys.version_info >= (3, 12):  # pragma: >=3.12 cover
            self.assertEqual(errors, ['W191:2:1', 'E901:3:28'])
        else:  # pragma: <3.12 cover
            self.assertEqual(errors, ['W191:2:1', 'E101:3:1'])
